@@ -34,7 +34,7 @@
 #include <orc/orc.h>
 #include <orc/orcdebug.h>
 
-
+#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -427,6 +427,30 @@ orc_test_random_bits (void *data, int n_bytes)
 #endif
 }
 
+static orc_uint64
+print_const_val_signed (const OrcVariable *const var)
+{
+  // in orcemulateopcodes the preference is given for "little" endian
+  switch (var->size) {
+    case 1:
+      printf (" %5" PRId8 " [%4" PRIu8 "]", (orc_int8)(var->value.x4[0] & 0xFF), (orc_uint8)(var->value.x4[0] & 0xFF));
+      return var->value.x4[0] & 0xFF;
+    case 2:
+      printf (" %6" PRId16 " [%5" PRIu16 "]", var->value.x4[0], (orc_uint16)var->value.x4[0]);
+      return var->value.x4[0];
+    case 4:
+      printf (" %11" PRId32 " [%10" PRIu32 "]", var->value.x2[0], (orc_uint32)var->value.x2[0]);
+      return var->value.x2[0];
+    case 8:
+      printf (" %20" PRId64 " [%20" PRIu64 "]", var->value.i,
+          (orc_uint64)var->value.i);
+      return var->value.i;
+    default:
+      printf (" ERROR!");
+      return -1;
+  }
+}
+
 static orc_uint64
 print_array_val_signed (OrcArray *array, int i, int j)
 {
@@ -435,23 +459,50 @@ print_array_val_signed (OrcArray *array, int i, int j)
 
   switch (array->element_size) {
     case 1:
-      printf(" %4d", *(orc_int8 *)ptr);
+      printf (" %5" PRId8 " [%4" PRIu8 "]", *(orc_int8 *)ptr, *(orc_uint8 *)ptr);
       return *(orc_int8 *)ptr;
     case 2:
-      printf(" %5d", *(orc_int16 *)ptr);
+      printf (" %6" PRId16 " [%5" PRIu16 "]", *(orc_int16 *)ptr, *(orc_uint16 *)ptr);
       return *(orc_int16 *)ptr;
     case 4:
-      printf(" %10d", *(orc_int32 *)ptr);
+      printf (" %11" PRId32 " [%10" PRIu32 "]", *(orc_int32 *)ptr, *(orc_uint32 *)ptr);
       return *(orc_int32 *)ptr;
     case 8:
-      printf(" 0x%08x%08x", (orc_uint32)((*(orc_uint64 *)ptr)>>32),
-          (orc_uint32)((*(orc_uint64 *)ptr)));
+      printf (" %20" PRId64 " [%20" PRIu64 "]", *(orc_int64 *)ptr,
+          *(orc_uint64 *)ptr);
       return *(orc_int64 *)ptr;
     default:
+      printf (" ERROR!");
       return -1;
   }
 }
 
+static void
+print_param_val_signed (const int *const var, const int *const var2,
+    const int size)
+{
+  switch (size) {
+    case 1:
+      printf (" %5" PRId8 " [%4" PRIu8 "]", *(orc_int8 *)var, *(orc_uint8 *)var);
+      break;
+    case 2:
+      printf (" %6" PRId16 " [%5" PRIu16 "]", *(orc_int16 *)var, *(orc_uint16 *)var);
+      break;
+    case 4:
+      printf (" %11" PRId32 " [%10" PRIu32 "]", *(orc_int32 *)var, *(orc_uint32 *)var);
+      break;
+    case 8:
+      {
+        const orc_uint64 v = ((orc_uint64)*var) << 32 | *(orc_uint32 *)var2;
+        printf (" %20" PRId64 " [%20" PRIu64 "]", (orc_int64)v, v);
+      }
+      break;
+    default:
+      printf (" ERROR!");
+      break;
+  }
+}
+
 #ifdef unused
 static orc_uint64
 print_array_val_unsigned (OrcArray *array, int i, int j)
@@ -487,17 +538,17 @@ print_array_val_hex (OrcArray *array, int i, int j)
 
   switch (array->element_size) {
     case 1:
-      printf(" %02x", *(orc_uint8 *)ptr);
+      printf(" %02" PRIx8, *(orc_uint8 *)ptr);
       return *(orc_int8 *)ptr;
     case 2:
-      printf(" %04x", *(orc_uint16 *)ptr);
+      printf(" %04" PRIx16, *(orc_uint16 *)ptr);
       return *(orc_int16 *)ptr;
     case 4:
-      printf(" %08x", *(orc_uint32 *)ptr);
+      printf(" %08" PRIx32, *(orc_uint32 *)ptr);
       return *(orc_int32 *)ptr;
     case 8:
-      printf(" 0x%08x%08x", (orc_uint32)((*(orc_uint64 *)ptr)>>32),
-          (orc_uint32)((*(orc_uint64 *)ptr)));
+      printf (" %20" PRId64 " [%20" PRIu64 "]", *(orc_int64 *)ptr,
+          *(orc_uint64 *)ptr);
       return *(orc_int64 *)ptr;
     default:
       return -1;
@@ -513,7 +564,7 @@ print_array_val_float (OrcArray *array, int i, int j)
   switch (array->element_size) {
     case 4:
       if (isnan(*(float *)ptr)) {
-        printf(" nan %08x", *(orc_uint32 *)ptr);
+        printf(" nan %08" PRIx32, *(orc_uint32 *)ptr);
       } else {
         printf(" %12.5g", *(float *)ptr);
       }
@@ -526,6 +577,48 @@ print_array_val_float (OrcArray *array, int i, int j)
   }
 }
 
+static void
+print_const_val_float (const OrcVariable *const var)
+{
+  switch (var->size) {
+    case 4:
+      if (isnan (var->value.x2f[0])) {
+        printf (" nan %08" PRIx32, var->value.x2[0]);
+      } else {
+        printf (" %12.5g", var->value.x2f[0]);
+      }
+      break;
+    case 8:
+      printf (" %12.5g", var->value.f);
+      break;
+    default:
+      printf (" ERROR");
+  }
+}
+
+static void
+print_param_val_float (const int *const var, const int *const var2,
+    const int size)
+{
+  switch (size) {
+    case 4:
+      if (isnan (*(float *)var)) {
+        printf (" nan %08" PRIx32, *var);
+      } else {
+        printf (" %12.5g", *(float *)var);
+      }
+      break;
+    case 8:
+      {
+        const orc_uint64 tmp = ((orc_uint64)*var2 << 32) | (orc_uint64)*var;
+        printf (" %12.5g", (double)tmp);
+      }
+      break;
+    default:
+      printf (" ERROR");
+  }
+}
+
 static float
 get_array_val_float (OrcArray *array, int i, int j)
 {
@@ -608,7 +701,7 @@ check_expected_failure (int flags, OrcProgram *p, OrcArray** src, OrcArray** des
 OrcTestResult
 orc_test_compare_output (OrcProgram *program)
 {
-  return orc_test_compare_output_full (program, 0);
+  return orc_test_compare_output_full_for_target (program, 0, NULL);
 }
 
 OrcTestResult
@@ -617,9 +710,32 @@ orc_test_compare_output_backup (OrcProgram *program)
   return orc_test_compare_output_full (program, ORC_TEST_FLAGS_BACKUP);
 }
 
-
 OrcTestResult
 orc_test_compare_output_full (OrcProgram *program, int flags)
+{
+  return orc_test_compare_output_full_for_target (program, flags, NULL);
+}
+
+static void
+dump_program (const OrcProgram *const program, const OrcTarget *const target)
+{
+  char fname[256] = { 0 };
+  snprintf (fname, 256, "%s-%s.S", program->name, target->name);
+  FILE *f = fopen (fname, "w");
+  ORC_ASSERT (f);
+  fprintf (f, "%s\n", program->asm_code);
+  fclose (f);
+
+  snprintf (fname, 256, "%s-%s.bin", program->name, target->name);
+  f = fopen (fname, "wb");
+  ORC_ASSERT (f);
+  fwrite (program->orccode->code, 1, program->orccode->code_size, f);
+  fclose (f);
+}
+
+
+OrcTestResult
+orc_test_compare_output_full_for_target (OrcProgram *program, int flags, const char *target_name)
 {
   OrcExecutor *ex;
   int n;
@@ -644,7 +760,7 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
     OrcTarget *target;
     unsigned int flags;
 
-    target = orc_target_get_default ();
+    target = orc_target_get_by_name (target_name);
     flags = orc_target_get_default_flags (target);
 
     result = orc_program_compile_full (program, target, flags);
@@ -657,6 +773,8 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
       ret = ORC_TEST_INDETERMINATE;
       goto out;
     }
+
+    dump_program(program, target);
   }
 
   if (program->constant_n > 0) {
@@ -725,7 +843,8 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
       orc_executor_set_stride (ex, i, src[i-ORC_VAR_S1]->stride);
     }
   }
-  ORC_DEBUG ("running");
+  ORC_DEBUG ("running %s\n", program->name);
+
   if (flags & ORC_TEST_FLAGS_BACKUP) {
     orc_executor_run_backup (ex);
   } else {
@@ -781,12 +900,33 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
         printf("%2d %2d:", i, j);
 
         for(l=ORC_VAR_S1;l<ORC_VAR_S1+8;l++){
-          if (program->vars[l].size > 0) {
+          switch (program->vars[l].vartype) {
+            case ORC_VAR_TYPE_SRC:
             if (flags & ORC_TEST_FLAGS_FLOAT) {
-              print_array_val_float (src[l-ORC_VAR_S1], i, j);
+              print_array_val_float (src[l - ORC_VAR_S1], i, j);
             } else {
-              print_array_val_hex (src[l-ORC_VAR_S1], i, j);
+              print_array_val_signed (src[l - ORC_VAR_S1], i, j);
             }
+              break;
+            case ORC_VAR_TYPE_CONST:
+              if (flags & ORC_TEST_FLAGS_FLOAT) {
+                print_const_val_float (program->vars + l);
+              } else {
+                print_const_val_signed (program->vars + l);
+              }
+              break;
+            case ORC_VAR_TYPE_PARAM:
+              if (flags & ORC_TEST_FLAGS_FLOAT) {
+                print_param_val_float (ex->params + l,
+                    ex->params + l + (ORC_VAR_T1-ORC_VAR_P1), program->vars[l].size);
+              } else {
+                print_param_val_signed (ex->params + l,
+                    ex->params + l + (ORC_VAR_T1-ORC_VAR_P1), program->vars[l].size);
+              }
+            case ORC_VAR_TYPE_DEST:
+            case ORC_VAR_TYPE_TEMP:
+            case ORC_VAR_TYPE_ACCUMULATOR:
+              break;
           }
         }
 
@@ -807,6 +947,7 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
               b = print_array_val_hex (dest_exec[l-ORC_VAR_D1], i, j);
               if (a != b) {
                 line_bad = TRUE;
+                n_lines_bad++;
               }
             }
           }
@@ -832,14 +973,33 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
           printf("%2d %2d:", i, j);
 
           for(k=0;k<ORC_N_VARIABLES;k++){
-            if (program->vars[k].name == NULL) continue;
-            if (program->vars[k].vartype == ORC_VAR_TYPE_SRC &&
-                program->vars[k].size > 0) {
-              if (flags & ORC_TEST_FLAGS_FLOAT) {
-                print_array_val_float (src[k-ORC_VAR_S1], i, j);
-              } else {
-                print_array_val_signed (src[k-ORC_VAR_S1], i, j);
-              }
+            switch (program->vars[k].vartype) {
+              case ORC_VAR_TYPE_SRC:
+                if (flags & ORC_TEST_FLAGS_FLOAT) {
+                  print_array_val_float (src[k - ORC_VAR_S1], i, j);
+                } else {
+                  print_array_val_signed (src[k - ORC_VAR_S1], i, j);
+                }
+                break;
+              case ORC_VAR_TYPE_CONST:
+                if (flags & ORC_TEST_FLAGS_FLOAT) {
+                  print_const_val_float (program->vars + k);
+                } else {
+                  print_const_val_signed (program->vars + k);
+                }
+                break;
+              case ORC_VAR_TYPE_PARAM:
+                if (flags & ORC_TEST_FLAGS_FLOAT) {
+                  print_param_val_float (ex->params + k,
+                      ex->params + k + (ORC_VAR_T1-ORC_VAR_P1), program->vars[k].size);
+                } else {
+                  print_param_val_signed (ex->params + k,
+                      ex->params + k + (ORC_VAR_T1-ORC_VAR_P1), program->vars[k].size);
+                }
+              case ORC_VAR_TYPE_DEST:
+              case ORC_VAR_TYPE_TEMP:
+              case ORC_VAR_TYPE_ACCUMULATOR:
+                break;
             }
           }
 
@@ -851,10 +1011,6 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
     }
   }
 
-  if (ret == ORC_TEST_FAILED) {
-    printf("%s", orc_program_get_asm_code (program));
-  }
-
   for(i=0;i<4;i++){
     if (dest_exec[i]) orc_array_free (dest_exec[i]);
     if (dest_emul[i]) orc_array_free (dest_emul[i]);
@@ -866,7 +1022,8 @@ orc_test_compare_output_full (OrcProgram *program, int flags)
   orc_executor_free (ex);
 
 out:
-  orc_program_reset (program);
+  if (!(flags & ORC_TEST_SKIP_RESET))
+    orc_program_reset (program);
 
   return ret;
 }
@@ -1062,6 +1219,8 @@ orc_test_performance_full (OrcProgram *program, int flags,
       orc_program_reset (program);
       return 0;
     }
+
+    dump_program(program, target);
   }
 
   if (program->constant_n > 0) {
@@ -1107,7 +1266,7 @@ orc_test_performance_full (OrcProgram *program, int flags,
     }
   }
 
-  ORC_DEBUG ("running");
+  ORC_DEBUG ("running %s\n", program->name);
   orc_profile_init (&prof);
   for(i=0;i<10;i++){
     orc_executor_set_n (ex, n);
